#!/bin/sh
### Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.

# This would work just as magical 'export PATH=$PATH' if invoked from MN on SN. Don't remove!
export PATH=/usr/local/bin:/bin:/usr/bin

script_name="$0"

read_conf()
{
	local psa_conf var val

	psa_conf=/etc/psa/psa.conf
	if test -r "$psa_conf"; then
		while read var val; do
			case "$var" in
				[A-Z]*) eval "$var=\"\$val\""
					;;
				*)
					;;
			esac
		done <"$psa_conf"
	else
		echo "psa not installed, use commandline parameters" >&2
	fi
}

locate_sources()
{
	local cf_path="$1"
	if ! [ -d "$cf_path" ]; then
		die "coldfusion not installed"
	fi
	SOURCE="`find "$cf_path" -name 'ApacheModule.zip' | head -1`"
	if ! [ -r "$SOURCE" ]; then
		SOURCE="`find "$cf_path" -name 'wsconfig.jar' | head -1`"
	fi

	if ! [ -r "$SOURCE" ]; then
		die "module sources not found in $cf_path"
	fi
}

die()
{
	echo "$script_name: $1" >&2
	exit 1
}

find_file()
{
	local var file list errmsg

	var=$1
	list=$2
	errmsg=$3
	if ! eval "test -x \"\$$var\""; then
		for file in $list; do
			if test -x "$file"; then
				eval "$var=\"\$file\""
				break
			fi
		done
	fi

	if ! eval "test -x \"\$$var\""; then
		die "$errmsg"
	fi
}

usage()
{
	echo "Usage:"
	echo "$script_name [-apxs <file>] [-httpd <file>] [-builddir <dir>] [-o <file>] { <cf_path> | <sources> }"
	echo " -apxs	 apache module builder. It must be installed in apache-devel package"
	echo " -httpd	 apache server executable"
	echo " -builddir temporary directory to build the module"
	echo " -o		 the resulting file name"
	echo " <cf_path> Coldfusion installation path"
	echo " <sources> Zip archive with module sources"
}

read_conf

while test $# -gt 0
do
	case "$1" in
		-apxs) APXS="$2"
			shift
		  ;;
		-httpd) HTTPD="$2"
			shift
		  ;;
		-builddir) BUILDDIR="$2"
			shift
		  ;;
		-o) TARGET="$2"
			shift
		  ;;
		--help|-h) usage
			exit
		  ;;
		*) 
			if [ -f "$1" ]; then
				SOURCE="$1"
			else
				locate_sources "$1"
			fi
		  ;;
	esac
	shift
done


find_file APXS "/usr/sbin/apxs2 /usr/bin/apxs2 /usr/sbin/apxs /usr/bin/apxs" \
		"apache builder not found. Probably apache-devel not installed."
find_file HTTPD "/usr/sbin/httpd2 /usr/sbin/httpd /usr/sbin/apache2" \
		"apache binary not found. Probably apache not installed."
find_file GCC "/usr/bin/gcc $CC" \
		"C compiler not found. Probably gcc not installed"

MAGIC=`"$HTTPD" -V| awk '/Server.s Module Magic Number:/{print $5}' FS='([[:space:]]|:)+'`
rc1=$?
VERSION=`"$HTTPD" -V| awk '/Server version:/{print $4,$5}' OFS='' FS='[ :/.]+'`
rc2=$?
if test "$rc1$rc2" != "00"; then
	die "cannot test apache installation"
fi
if test "$VERSION" != "20" -a "$VERSION" != "22"; then
	die "bad apache version $VERSION"
fi

SONAME="mod_jrun$VERSION.so"

if test -z "$BUILDDIR"; then
	BUILDDIR=`mktemp -d /tmp/build-mod_jrun$VERSION-XXXXXXXX` || die "cannot create builddir"
else
	mkdir -p $BUILDDIR || die "cannot create builddir $BUILDDIR"
fi

if test -z "$TARGET"; then
	if test -n "$PRODUCT_ROOT_D"; then
		TARGET="$PRODUCT_ROOT_D/admin/share/$SONAME.$MAGIC"
	else
		TARGET="$PWD/$SONAME.$MAGIC"
	fi
fi

TARGETDIR=`dirname "$TARGET"`

if test -z "$SOURCE"; then
	echo "sources location is not defined"
	usage >&2
	exit 1
fi

mkdir -p "$TARGETDIR"
case "$SOURCE" in
*.zip)
	unzip -d "$BUILDDIR" "$SOURCE"
	cd "$BUILDDIR"
	;;
*.jar)
	unzip -d "$BUILDDIR" "$SOURCE" 'connectors/src/*'
	cd "$BUILDDIR/connectors/src"
	;;
*)
	die "unknown source archive $SOURCE"
	;;
esac
chmod 644 *
if [ -e jrun_utils.c ]; then
	# coldfusion 7.0
	sources="mod_jrun$VERSION.c  \
		jrun_maptable_impl.c  jrun_property.c  jrun_session.c platform.c \
		jrun_mutex.c jrun_proxy.c jrun_utils.c"
else
	# coldfusion 6.1
	sources="mod_jrun$VERSION.c  \
		jrun_maptable_impl.c  jrun_property.c  jrun_session.c platform.c \
		jrun_mutex.c  jrun_proxy.c jrun_ssl.c"
fi
"$APXS" -c -Wc,-w -n jrun$VERSION -S LIBEXECDIR="$TARGETDIR" $sources || die "build failed"
"$APXS" -i -n jrun$VERSION -S LIBEXECDIR="$TARGETDIR" mod_jrun$VERSION.la || die "build failed"

mv "$TARGETDIR/$SONAME" "$TARGET"

rm -rf "$BUILDDIR"
